import { visualizationColors } from 'config';
import React, { useRef, useEffect, useState } from 'react';
import Plot from 'react-plotly.js';
import { Tooltip, TooltipContent, TooltipTrigger } from '@graphpolaris/shared/lib/components/tooltip';

const getCSSVariableHSL = (varName: string) => {
  const rootStyles = getComputedStyle(document.documentElement);
  const hslValue = rootStyles.getPropertyValue(varName).trim().replace('deg', '');
  return `hsl(${hslValue})`;
};
export const plotTypeOptions = ['bar', 'scatter', 'line', 'histogram', 'pie'] as const;

export interface CustomChartPlotlyProps {
  xAxisData: string[] | number[];
  plotType: (typeof plotTypeOptions)[number];
  title: string;
  showAxis: boolean;
  yAxisData: string[] | number[];
  xAxisLabel?: string;
  yAxisLabel?: string;
}

export const getPlotData = (
  xAxisData: (string | number)[],
  plotType: (typeof plotTypeOptions)[number],
  yAxisData?: (string | number)[],
): Partial<Plotly.PlotData>[] => {
  const mainColors = visualizationColors.GPCat.colors[14];

  const primaryColor = getCSSVariableHSL('--clr-sec--400');
  let xValues: (string | number)[] = [];
  let yValues: (string | number)[] = [];

  if (plotType === 'scatter' || plotType === 'line') {
    if (xAxisData.length != 0 && yAxisData && yAxisData.length != 0) {
      xValues = xAxisData;
      yValues = yAxisData;
    } else if (xAxisData.length != 0 && yAxisData && yAxisData.length == 0) {
      xValues = xAxisData;
      yValues = xAxisData.map((_, index) => index + 1);
    } else if (xAxisData.length == 0 && yAxisData && yAxisData.length != 0) {
      xValues = yAxisData.map((_, index) => index + 1);
      yValues = yAxisData;
    } else if (xAxisData.length == 0 && yAxisData && yAxisData.length == 0) {
    } else {
    }
  } else {
    xValues = xAxisData;
    yValues = xAxisData.map((_, index) => index + 1);
  }

  switch (plotType) {
    case 'bar':
      return [
        {
          type: 'bar',
          x: xValues,
          y: yValues,
          marker: { color: primaryColor },
        },
      ];
    case 'scatter':
      return [
        {
          type: 'scatter',
          x: xValues,
          y: yValues,
          mode: 'markers',
          marker: { color: primaryColor, size: 12 },
        },
      ];
    case 'line':
      return [
        {
          type: 'scatter',
          x: xValues,
          y: yValues,
          mode: 'lines',
          line: { color: primaryColor },
        },
      ];
    case 'histogram':
      return [
        {
          type: 'histogram',
          x: xAxisData,
          marker: { color: primaryColor },
        },
      ];
    case 'pie':
      return [
        {
          type: 'pie',
          labels: xValues.map(String),
          values: xAxisData,
          marker: { colors: mainColors },
        },
      ];

    default:
      return [];
  }
};

export const CustomChartPlotly: React.FC<CustomChartPlotlyProps> = ({
  xAxisData,
  plotType,
  title,
  showAxis,
  yAxisData,
  xAxisLabel,
  yAxisLabel,
}) => {
  const internalRef = useRef<HTMLDivElement>(null);
  const [divSize, setDivSize] = useState({ width: 0, height: 0 });
  const [hoveredPoint, setHoveredPoint] = useState<{ left: number; top: number; xValue: number; yValue: number } | null>(null);

  useEffect(() => {
    const handleResize = () => {
      if (internalRef.current) {
        const { width, height } = internalRef.current.getBoundingClientRect();
        setDivSize({ width, height });
      }
    };

    handleResize();
    window.addEventListener('resize', handleResize);
    if (internalRef.current) {
      new ResizeObserver(handleResize).observe(internalRef.current);
    }

    return () => {
      window.removeEventListener('resize', handleResize);
    };
  }, []);

  const handleHover = (event: any) => {
    const { points } = event;
    if (points.length) {
      const point = points[0];
      const plotRect = internalRef.current?.getBoundingClientRect();

      if (plotRect) {
        const xIndex = point.xaxis.d2p(point.x);
        const yIndex = point.yaxis.d2p(point.y);

        setHoveredPoint({
          left: plotRect.left + xIndex,
          top: plotRect.top + yIndex,
          xValue: point.x,
          yValue: point.y,
        });
      }
    }
  };

  const handleUnhover = () => {
    setHoveredPoint(null);
  };

  return (
    <div className="h-full w-full flex items-center justify-center overflow-hidden relative" ref={internalRef}>
      <Plot
        data={getPlotData(xAxisData, plotType, yAxisData)}
        config={{
          responsive: true,
          scrollZoom: false,
          displayModeBar: false,
          displaylogo: false,
        }}
        layout={{
          width: divSize.width,
          height: divSize.height,
          title: title,
          dragmode: false,
          font: {
            family: 'Inter, sans-serif',
            size: 12,
            color: '#374151',
          },
          xaxis: {
            title: showAxis ? (xAxisLabel ? xAxisLabel : '') : '',
            showgrid: false,
            visible: showAxis,
            showline: true,
            zeroline: false,
          },
          yaxis: {
            title: showAxis ? (yAxisLabel ? yAxisLabel : '') : '',
            showgrid: false,
            visible: showAxis,
            showline: true,
            zeroline: false,
          },
          hoverlabel: {
            bgcolor: 'rgba(255, 255, 255, 0.8)',
            bordercolor: 'rgba(0, 0, 0, 0.2)',
            font: {
              family: 'Inter, sans-serif',
              size: 14,
              color: '#374151',
            },
          },
        }}
        onHover={handleHover}
        onUnhover={handleUnhover}
      />
      {/* 
      {hoveredPoint && (
        <div>
          <Tooltip open={true} showArrow={true}>
            <TooltipTrigger />
            <TooltipContent
              style={{
                position: 'absolute',
                left: hoveredPoint.left,
                top: hoveredPoint.top,
                transform: 'translate(-50%, 0%)',
              }}
            >
              <div>
                <strong>{hoveredPoint.xValue}</strong>: {hoveredPoint.yValue}
              </div>
            </TooltipContent>
          </Tooltip>
        </div>
      )}
        */}
    </div>
  );
};
